@@ -1,30 +1,29 @@
+import re
+import time
+
+from git.compat import (
+    PY3,
+    xrange,
+    string_types,
+    defenc
+)
+from git.objects.util import (
+    parse_date,
+    Serializable,
+    altz_to_utctz_str,
+)
 from git.util import (
     Actor,
     LockedFD,
     LockFile,
     assure_directory_exists,
     to_native_path,
-)
-
-from gitdb.util import (
     bin_to_hex,
-    join,
-    file_contents_ro_filepath,
+    file_contents_ro_filepath
 )
 
-from git.objects.util import (
-    parse_date,
-    Serializable,
-    altz_to_utctz_str,
-)
-from git.compat import (
-    xrange,
-    string_types,
-    defenc
-)
+import os.path as osp
 
-import time
-import re
 
 __all__ = ["RefLog", "RefLogEntry"]
 
@@ -32,16 +31,29 @@ __all__ = ["RefLog", "RefLogEntry"]
 class RefLogEntry(tuple):
 
     """Named tuple allowing easy access to the revlog data fields"""
-    _fmt = "%s %s %s <%s> %i %s\t%s\n"
     _re_hexsha_only = re.compile('^[0-9A-Fa-f]{40}$')
-    __slots__ = tuple()
+    __slots__ = ()
 
     def __repr__(self):
         """Representation of ourselves in git reflog format"""
+        res = self.format()
+        if PY3:
+            return res
+        # repr must return a string, which it will auto-encode from unicode using the default encoding.
+        # This usually fails, so we encode ourselves
+        return res.encode(defenc)
+
+    def format(self):
+        """:return: a string suitable to be placed in a reflog file"""
         act = self.actor
         time = self.time
-        return self._fmt % (self.oldhexsha, self.newhexsha, act.name, act.email,
-                            time[0], altz_to_utctz_str(time[1]), self.message)
+        return u"{} {} {} <{}> {!s} {}\t{}\n".format(self.oldhexsha,
+                                                     self.newhexsha,
+                                                     act.name,
+                                                     act.email,
+                                                     time[0],
+                                                     altz_to_utctz_str(time[1]),
+                                                     self.message)
 
     @property
     def oldhexsha(self):
@@ -99,7 +111,7 @@ class RefLogEntry(tuple):
         newhexsha = info[41:81]
         for hexsha in (oldhexsha, newhexsha):
             if not cls._re_hexsha_only.match(hexsha):
-                raise ValueError("Invalid hexsha: %s" % hexsha)
+                raise ValueError("Invalid hexsha: %r" % (hexsha,))
             # END if hexsha re doesn't match
         # END for each hexsha
 
@@ -120,7 +132,7 @@ class RefLog(list, Serializable):
     of the head in question. Custom query methods allow to retrieve log entries
     by date or by other criteria.
 
-    Reflog entries are orded, the first added entry is first in the list, the last
+    Reflog entries are ordered, the first added entry is first in the list, the last
     entry, i.e. the last change of the head or reference, is last in the list."""
 
     __slots__ = ('_path', )
@@ -170,7 +182,7 @@ class RefLog(list, Serializable):
             instance would be found. The path is not guaranteed to point to a valid
             file though.
         :param ref: SymbolicReference instance"""
-        return join(ref.repo.git_dir, "logs", to_native_path(ref.path))
+        return osp.join(ref.repo.git_dir, "logs", to_native_path(ref.path))
 
     @classmethod
     def iter_entries(cls, stream):
@@ -189,13 +201,14 @@ class RefLog(list, Serializable):
                 return
             yield new_entry(line.strip())
         # END endless loop
+        stream.close()
 
     @classmethod
     def entry_at(cls, filepath, index):
         """:return: RefLogEntry at the given index
         :param filepath: full path to the index file from which to read the entry
         :param index: python list compatible index, i.e. it may be negative to
-            specifiy an entry counted from the end of the list
+            specify an entry counted from the end of the list
 
         :raise IndexError: If the entry didn't exist
 
@@ -232,7 +245,7 @@ class RefLog(list, Serializable):
         try:
             self._serialize(fp)
             lfd.commit()
-        except:
+        except Exception:
             # on failure it rolls back automatically, but we make it clear
             lfd.rollback()
             raise
@@ -258,19 +271,19 @@ class RefLog(list, Serializable):
             raise ValueError("Shas need to be given in binary format")
         # END handle sha type
         assure_directory_exists(filepath, is_file=True)
+        first_line = message.split('\n')[0]
         committer = isinstance(config_reader, Actor) and config_reader or Actor.committer(config_reader)
         entry = RefLogEntry((
             bin_to_hex(oldbinsha).decode('ascii'),
             bin_to_hex(newbinsha).decode('ascii'),
-            committer, (int(time.time()), time.altzone), message
+            committer, (int(time.time()), time.altzone), first_line
         ))
 
         lf = LockFile(filepath)
         lf._obtain_lock_or_raise()
-
         fd = open(filepath, 'ab')
         try:
-            fd.write(repr(entry).encode(defenc))
+            fd.write(entry.format().encode(defenc))
         finally:
             fd.close()
             lf._release_lock()
@@ -295,7 +308,7 @@ class RefLog(list, Serializable):
 
         # write all entries
         for e in self:
-            write(repr(e).encode(defenc))
+            write(e.format().encode(defenc))
         # END for each entry
 
     def _deserialize(self, stream):
